#!/bin/sh
#
# Usage: install-ciphire.sh [system-dir]
# Version: 0.10.51
# This script installs the Ciphire Mail binary distribution.
# Copyright (c) 2003-2005 Ciphire Labs

CITARGET_ROOT_DEFAULT=/usr/local/ciphire
CITARGET_DEFAULT=$HOME/ciphire
USERID=`id -u`
GROUPID=`id -g`
DATE=`date`
CN=`basename $0`
EXPERT=
XSUPPORT=1
CIDIR="./bin"

cd `dirname $0`

if [ -e "setupdata/libciphire.sh" ]
then
  . setupdata/libciphire.sh
else
  echo "ERROR: libciphire.sh not found!"
  exit 1
fi

writelog "Starting installation [`uname -a`]"

if set_popecho
then
  HAVEDISPLAY=1
else
  HAVEDISPLAY=0
fi

for DL in /usr/lib/libdl.so /lib/libdl.so /lib/libdl.so.2 /usr/lib/libdl.so.2 ""
do
  test -f $DL || continue
  LD_PRELOAD=$DL:connectors/ciphire-redir.so /bin/true 2>/dev/null || continue
  PRELOAD_VALUE=$DL:connectors/ciphire-redir.so
  break
done
if ! LD_PRELOAD=$PRELOAD_VALUE /bin/true 2>/dev/null
then
  writelog "Preload check failed. Value=$PRELOAD_VALUE"
  popecho error "Ciphire Mail requires glibc version 2.3 or higher.\nInstallation failed!\n" error
fi
unset LD_PRELOAD
 
popecho yesno "Installing Ciphire Mail binary distribution 0.10.51 for platform linux-i686.\n\nWould you like to perform a default installation?\n\n - Answering 'yes' will install all files in their default location and activate Ciphire Mail whenever you login (normal mode, recommended).\n - Answering 'no' allows you to choose file locations and certain configuration settings of Ciphire Mail (expert mode).\n" qm
if [ "$?" = "1" ]
then
  EXPERT=1
fi

popecho license "Do you accept the terms as set forth in the license?\n" qm


if [ "x$1" = "x" ]
then
 if [ ! $EXPERT ]
  then
    CUSTOM_SYS=
  else
    popecho input "Where would you like to install the Ciphire Mail system files?\nIf required, the installer will ask for the administrator password.\n" qm $CITARGET_ROOT_DEFAULT
    CUSTOM_SYS=$INPUT_VALUE
  fi
  if [ "x$CUSTOM_SYS" = "x" ] 
  then
    CITARGET=$CITARGET_ROOT_DEFAULT
  else
    CITARGET=$CUSTOM_SYS
  fi
else
  CITARGET=$1
fi

if [ $EXPERT ]
then
  popecho cont "System files will be installed in '$CITARGET'.\n" info
  popecho yesno "Would you like to install X11 session support of Ciphire Mail?\nThis ensures that the Ciphire Redirector is activated when your X11\nsession/window is started. Files in your /etc/X11 directory are to be\nadded or modified and therefore administrator privileges are required.\nIf unsure say 'yes'.\n" qm
  if [ "$?" = "1" ]
  then
    XSUPPORT=0
  fi
fi

writelog "Going to install in $CITARGET [XSUPPORT=$XSUPPORT]"

if [ "$USERID" = "0" ]
then
  writelog "Invoking ciphire-systemsetup as root (direct)"
  ./bin/ciphire-systemsetup $CITARGET $XSUPPORT $USERID $GROUPID $HAVEDISPLAY
else
  NEEDROOT=1
  if [ -e "$CITARGET" ]
  then
    if [ -w "$CITARGET" -a $XSUPPORT = "0" ]
    then
      NEEDROOT=
    fi
  else
    if [ -w "`dirname $CITARGET`" -a $XSUPPORT = "0" ]
    then
      NEEDROOT=
    fi
  fi
  if [ $NEEDROOT ]
  then
    writelog "Need root for ciphire-systemsetup"
    popecho cont "The next installation step requires root (administrator) privileges.\n" info
    ./bin/ciphire-systemsetup $CITARGET $XSUPPORT $USERID $GROUPID $HAVEDISPLAY $NEEDROOT
  else
    writelog "Invoking ciphire-systemsetup as user"
    ./bin/ciphire-systemsetup $CITARGET $XSUPPORT $USERID $GROUPID $HAVEDISPLAY
  fi
fi

if [ "$?" != "0" ]
then
  popecho error "Installation of Ciphire Mail failed!\n" error
fi
 
if [ "$USERID" != "0" ]
then
  if [ $EXPERT ]
  then
    writelog "Invoking ciphire-setup 1 $HAVEDISPLAY"
    $CITARGET/ciphire-setup 1 $HAVEDISPLAY
  else
    writelog "Invoking ciphire-setup 0 $HAVEDISPLAY"
    $CITARGET/ciphire-setup 0 $HAVEDISPLAY
  fi
else
  popecho yesno "Would you like to activate Ciphire Mail for any users?\n" qm
  if [ "$?" = "0" ]
  then
    popecho input "For which users (accounts) would you like to activate Ciphire Mail?\n(Please note that for every user a default setup\nwill be performed where Ciphire Mail is activated\neither in the user's KDE or Gnome autostart folder.)\n\nEnter each account name in the following line and separate each\naccount with a space (e.g., 'user1 user2 user3'):\n" ""
    NEWUSERS=$INPUT_VALUE
    if [ "$NEWUSERS" ]
    then
      writelog "Processing user accounts: $NEWUSERS"
      PROCESSED_ACCOUNTS=1
      PROCESSED_OUTPUT="Processed accounts:"
      for i in `echo "$NEWUSERS"`
      do
        UHOME=`su - $i -c "echo ~" 2>/dev/null`
        if [ ! "x$UHOME" = "x" ]
        then
          writelog "Invoking ciphire-setup 0 0 [user=$i]"
          su - $i -c "$CITARGET/ciphire-setup 0 0" >/dev/null
          PROCESSED_OUTPUT="${PROCESSED_OUTPUT}\n- ${i}: Ciphire Mail user configuration installed or updated."
        else
          PROCESSED_OUTPUT="${PROCESSED_OUTPUT}\n- ${i}: Error, user does not exist."
        fi
      done
      popecho nop "${PROCESSED_OUTPUT}\n" info
      writelog "Finished processing user accounts."
    fi
  fi

  FINISH_HEADER="\nInstallation of Ciphire Mail completed.\n(Logfile: /tmp/install-ciphire.log)\n"
  FINISH_X1="\nX11 session/window manager support has been installed.\nTo activate it, any running X11 session has to be\nrestarted, i.e., a relogin to the account is required!\nIf unsure, just restart your computer system.\n"
  FINISH_X0="\nX11 session manager support has NOT been installed.\nPlease make sure that users always start their email\napplication with the ciphire-ctl script. Otherwise\nCiphire Mail won't work properly!\n"
  FINISH_P1="\nUsers for which Ciphire Mail has not been activated\nmay do this at any time by invoking the setup script\n$CITARGET/ciphire-setup\n"
  FINISH_P0="\nIndividual users have to invoke the setup script\n$CITARGET/ciphire-setup\nbefore using Ciphire Mail.\n"
  FINISH_FOOTER="\nThank you for using Ciphire.\n"

  if [ "x$XSUPPORT" = "x1" ]
  then
    FINISH="${FINISH_HEADER}${FINISH_X1}"
  else
    FINISH="${FINISH_HEADER}${FINISH_X0}"
  fi
  if [ "x$PROCESSED_ACCOUNTS" = "x1" ]
  then
    FINISH="${FINISH}${FINISH_P1}"
  else
    FINISH="${FINISH}${FINISH_P0}"
  fi
  FINISH="${FINISH}${FINISH_FOOTER}"
  popecho nop "$FINISH" info
fi

writelog "Finished installation."
